package bank.client;

import java.rmi.NoSuchObjectException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import bank.common.Account;
import bank.common.Bank;
import bank.common.Customer;
import bank.common.RemoteAccountEvent;
import bank.common.RemoteAccountListener;

public class BankClient_Callback {

	@SuppressWarnings("unused")
	public static void main(String[] args) {
		Customer cust1 = null; 
		Customer cust2 = null; 
		Account acc1 = null; 
		Account acc2 = null; 
		AccountWatcher acc1Watcher = null; 
		AccountWatcher acc2Watcher = null; 
		try {
			Registry reg = LocateRegistry.getRegistry("localhost", Bank.PORT);
			Bank bank = (Bank) reg.lookup("Bank");

			cust1 = bank.createCustomer("Berger");
			cust2 = bank.createCustomer("Maier");

			acc1 = bank.createAccount("Berger");
			acc2 = bank.createAccount("Maier");
			
			acc1Watcher = new AccountWatcher("Berger");
			acc2Watcher = new AccountWatcher("Maier");
			acc1.addRemoteAccountListener(acc1Watcher);
			acc2.addRemoteAccountListener(acc2Watcher);
			
			acc1.deposit(10000);
			acc2.deposit(20000);
			
			boolean success = bank.transfer(acc1.getId(), acc2.getId(), 3000);
			
			
		} catch (Exception e) {
			System.out.println("Trouble: " + e);
		} finally {
			try { acc1.removeRemoteAccountListener(acc1Watcher); } catch (Exception e) {}
			try { acc2.removeRemoteAccountListener(acc2Watcher); } catch (RemoteException e) { }
			try { UnicastRemoteObject.unexportObject(acc1Watcher, true); } catch (NoSuchObjectException e) { }
			try { UnicastRemoteObject.unexportObject(acc2Watcher, true);} catch (NoSuchObjectException e) {	}		
		}
	}
	
	@SuppressWarnings("serial")
	private static class AccountWatcher 
					extends UnicastRemoteObject implements RemoteAccountListener {
		private final String name; 
		public AccountWatcher(String name) throws RemoteException {
			super();
			this.name = name;
		}

		@Override
		public void accountChanged(RemoteAccountEvent e) throws RemoteException {
			System.out.println("Account of " + name + " changed!");
		}
		
	};

}
