package bank.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import bank.common.Account;
import bank.common.Customer;
import bank.common.RemoteAccountEvent;
import bank.common.RemoteAccountListener;

@SuppressWarnings("serial")
public class AccountImpl extends UnicastRemoteObject implements Account {

	private static ExecutorService ex = Executors.newCachedThreadPool(); 
	private final int id;
	private final String customer;
	private long balance;
	private final List<RemoteAccountListener> listeners; 

	public AccountImpl(int id, String customer) throws RemoteException {
		super();
		this.id = id;
		this.customer = customer;
		this.balance = 0L;
		this.listeners = new CopyOnWriteArrayList<>();
	}

	@Override
	public int getId() throws RemoteException {
		return id;
	}

	@Override
	public Customer getCustomer() throws RemoteException {
		return BankImpl.getInstance().getCustomer(customer);
	}

	@Override
	public synchronized long getBalance() throws RemoteException {
		return balance;
	}

	@Override
	public long withdraw(long diff) throws RemoteException {
		synchronized (this) {
			balance = balance - diff;
			fireAccountChanged();
		}
		return diff;
	}

	@Override
	public void deposit(long diff) throws RemoteException {
		synchronized (this) {
			balance = balance + diff;
			fireAccountChanged();
		}
	}

	@Override
	public void addRemoteAccountListener(RemoteAccountListener l) throws RemoteException {
		listeners.add(l);
	}

	@Override
	public void removeRemoteAccountListener(RemoteAccountListener l) throws RemoteException {
		listeners.remove(l);
	}
	
	private void fireAccountChanged() throws RemoteException {
		ex.submit(() -> {
				RemoteAccountEvent evt = new RemoteAccountEvent("Account" + id, this.id, this.balance); 
				for (RemoteAccountListener l: listeners) {
					try {
						l.accountChanged(evt);
					} catch (RemoteException e) {
						System.out.println("Unable to fire event " + e.getMessage());
					}
				}
			});
		
	}

}
