package bank.server;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;
import java.util.Map;

import bank.common.Account;
import bank.common.AccountInfo;
import bank.common.Bank;
import bank.common.Customer;

@SuppressWarnings("serial")
public class BankImpl extends UnicastRemoteObject implements Bank {
	private static BankImpl instance;

	public synchronized static Bank getInstance() {
		if (instance == null) {
			try {
				instance = new BankImpl();
			} catch (RemoteException e) {
			}
		}
		return instance;
	}

	private static int id = 0;
	private final Map<Integer, Account> accounts;
	private final Map<String, CustomerImpl> customers;

	private BankImpl() throws RemoteException {
		accounts = new HashMap<Integer, Account>();
		customers = new HashMap<String, CustomerImpl>();
	}

	@Override
	public synchronized Customer getCustomer(String name) throws RemoteException {
		return customers.get(name);
	}

	@Override
	public synchronized Account getAccount(int id) throws RemoteException {
		return accounts.get(id);
	}

	@Override
	public synchronized AccountInfo getAccountInfo(int accountNumber) throws RemoteException {
		Account account = getAccount(accountNumber);
		if (account != null) {
			return new AccountInfo(accountNumber, account.getCustomer().getName(), account.getBalance());
		} else {
			return null; 
		}
	}

	@Override
	public synchronized Customer createCustomer(String name) throws RemoteException {
		CustomerImpl c = new CustomerImpl(name);
		customers.put(name, c);
		return customers.get(name);
	}

	@Override
	public synchronized Account createAccount(String customerName) throws RemoteException {
		id++;
		CustomerImpl customer = customers.get(customerName);
		Account a = new AccountImpl(id, customerName);
		accounts.put(id, a);
		customer.accounts.add(a);
		return a;
	}

	@Override
	public synchronized boolean transfer(int from, int to, long amount) throws RemoteException {
		Account aFrom = getAccount(from);
		Account aTo = getAccount(to);
		aFrom.withdraw(amount);
		aTo.deposit(amount);
		return true;
	}

	// NOT A GOOD METHOD
	@Override
	public synchronized boolean transfer(Account from, Account to, long amount) throws RemoteException {
		if (!accounts.containsValue(to)) {
			return false;   // does not work as expected!! 
		}
		if (from == to) {
			return false;   // does not work as expected!! 
		}
		// must work with ids anyway but will retrieve ids from client which gets it from server 
		return transfer(from.getId(), to.getId(), amount);
	}

}
